@@ -29,7 +29,7 @@ from .qt import *
 import pandas as pd
 from .core import DataFrameModel, DataFrameTable, DataFrameWidget
 from .plotting import PlotViewer
-from . import util, data, core, dialogs, widgets, plotting
+from . import util, core, dialogs, widgets, plotting
 
 homepath = os.path.expanduser("~")
 module_path = os.path.dirname(os.path.abspath(__file__))
@@ -46,6 +46,22 @@ border-radius: 4px;
 }
 """
 
+dockstyle = '''
+    QDockWidget {
+        max-width:240px;
+    }
+    QDockWidget::title {
+        background-color: lightblue;
+    }
+    QScrollBar:vertical {
+         width: 15px;
+         margin: 1px 0 1px 0;
+     }
+    QScrollBar::handle:vertical {
+         min-height: 20px;
+     }
+'''
+
 class Application(QMainWindow):
     def __init__(self, project_file=None, csv_file=None, excel_file=None):
 
@@ -72,6 +88,7 @@ class Application(QMainWindow):
         self.setCentralWidget(self.main)
 
         #plot docks
+        plotting.update_colormaps()
         self.addDockWidgets()
         self.statusbar = QStatusBar()
         self.setStatusBar(self.statusbar)
@@ -85,6 +102,7 @@ class Application(QMainWindow):
         self.recent_files = ['']
         self.recent_urls = []
         self.scratch_items = {}
+        self.openplugins = {}
 
         self.loadSettings()
         self.setTheme()
@@ -109,21 +127,6 @@ class Application(QMainWindow):
     def addDockWidgets(self):
         """Add plot dialogs to dock"""
 
-        dockstyle = '''
-            QDockWidget {
-                max-width:240px;
-            }
-            QDockWidget::title {
-                background-color: lightblue;
-            }
-            QScrollBar:vertical {
-                 width: 15px;
-                 margin: 1px 0 1px 0;
-             }
-            QScrollBar::handle:vertical {
-                 min-height: 20px;
-             }
-        '''
         style = '''
             QWidget {
                 font-size: 12px;
@@ -162,7 +165,6 @@ class Application(QMainWindow):
                                 style=style)
             area.setWidget(dialog)
             self.addDockWidget(QtCore.Qt.RightDockWidgetArea, dock)
-            #dock.setAllowedAreas(Qt.AllDockWidgetAreas)
             self.plotwidgets[name] = widgets
             docks[name] = dock
 
@@ -175,6 +177,28 @@ class Application(QMainWindow):
             action = self.docks[name].toggleViewAction()
             self.dock_menu.addAction(action)
             action.setCheckable(True)
+        #series dock
+        #self.addSeriesDock()
+        return
+
+    '''def addSeriesDock(self):
+        """Add series dock. Updated by plotter when selections changed."""
+
+        dock = QDockWidget('series')
+        dock.setStyleSheet("QDockWidget::title {background-color: #99ccff;}")
+        area = QScrollArea()
+        area.setWidgetResizable(True)
+        dock.setWidget(area)
+        self.seriesarea = area
+        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, dock)
+        return'''
+
+    def replot(self):
+        """Plot current"""
+
+        w = self.getCurrentTable()
+        pf = w.pf
+        pf.replot()
         return
 
     def tabSelected(self, index):
@@ -187,6 +211,8 @@ class Application(QMainWindow):
         #print (table.pf)
         #get plot options and update widgets
         self.updatePlotWidgets(table)
+        #update any plugins to use the current table if needed
+        self.updatePlugins()
         return
 
     def updatePlotWidgets(self, table):
@@ -194,17 +220,18 @@ class Application(QMainWindow):
 
         for key in self.plotwidgets:
             opts = table.pf.opts[key]
-            if key == 'series':
-                df = table.table.model.df
-                #opts.widgets = self.plotwidgets[key]
-                area = self.docks[key].widget()
-                opts.updateWidgets(area, df)
-                self.plotwidgets[key] = opts.widgets
-            else:
-                opts.widgets = self.plotwidgets[key]
-                opts.updateWidgets()
+            opts.widgets = self.plotwidgets[key]
+            opts.updateWidgets()
         table.pf.updateData()
+        return
 
+    def updatePlugins(self):
+        """Update table for a plugin if it needs it"""
+
+        for o in self.openplugins:
+            print (o)
+            w = self.getCurrentTable()
+            self.openplugins[o].table = w
         return
 
     def startLogging(self):
@@ -213,7 +240,8 @@ class Application(QMainWindow):
         import logging
         if platform.system() == 'Windows':
             path = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.ConfigLocation)
-            print (path)
+            if not os.path.exists(path):
+                os.makedirs(path)
         else:
             path = os.path.dirname(self.settings.fileName())
         self.logfile = os.path.join(path, 'error.log')
@@ -225,7 +253,7 @@ class Application(QMainWindow):
 
         for s in core.defaults:
             k = s.lower()
-            if k not in self.settings.childKeys():                
+            if k not in self.settings.childKeys():
                 self.settings.setValue(k,core.defaults[s])
 
     def loadSettings(self):
@@ -427,10 +455,12 @@ class Application(QMainWindow):
         self.sheet_menu.addAction(icon, 'Add', self.addSheet)
         self.sheet_menu.addAction('Rename', self.renameSheet)
         icon = QIcon(os.path.join(iconpath,'copy.png'))
-        self.sheet_menu.addAction(icon, '&Copy', self.copySheet)
-        self.sheet_menu.addAction('Concat', self.concatSheets)
+        #self.sheet_menu.addAction(icon, 'Copy', self.copySheet)
+        self.sheet_menu.addAction(icon, 'Duplicate', self.duplicateSheet)
+        self.sheet_menu.addAction('Join', self.concatSheets)
         icon = QIcon(os.path.join(iconpath,'merge.png'))
         self.sheet_menu.addAction(icon, 'Merge', self.mergeSheets)
+        self.sheet_menu.addAction('Clear All', self.clearSheets)
 
         self.tools_menu = QMenu('Tools', self)
         icon = QIcon(os.path.join(iconpath,'tableinfo.png'))
@@ -477,8 +507,10 @@ class Application(QMainWindow):
         self.menuBar().addSeparator()
         self.menuBar().addMenu(self.help_menu)
         self.help_menu.addAction('View Error Log', self.showErrorLog)
+        url = 'https://tablexplore.readthedocs.io/en/latest/'
+        self.help_menu.addAction('Online Help', lambda: self.open_url(url))
         icon = QIcon(os.path.join(iconpath,'logo.png'))
-        self.help_menu.addAction(icon, '&About', self.about)
+        self.help_menu.addAction(icon, 'About', self.about)
 
         #plot shortcut
         self.plotshc = QShortcut(QKeySequence('Ctrl+P'), self)
@@ -903,7 +935,6 @@ class Application(QMainWindow):
         sheet.addWidget(pf)
         sheet.setSizes((500,1000))
 
-        #pf.applyPlotoptions()
         #reload attributes of table and plotter if present
         if meta != None:
             self.loadMeta(dfw, meta)
@@ -913,8 +944,7 @@ class Application(QMainWindow):
                 pf.hide()
 
         self.updatePlotWidgets(dfw)
-        #set default style
-        #pf.opts['format'].setWidgetValue('style', core.PLOTSTYLE)
+        self.updatePlugins()
         self.tabs.setCurrentIndex(idx)
         return
 
@@ -934,6 +964,8 @@ class Application(QMainWindow):
     def renameSheet(self):
         """Rename the current sheet"""
 
+        if len(self.sheets) == 0:
+            return
         index = self.tabs.currentIndex()
         name = self.tabs.tabText(index)
         new, ok = QInputDialog.getText(self, 'New name', 'Name:',
@@ -948,21 +980,26 @@ class Application(QMainWindow):
             self.tabs.setTabText(index, new)
         return
 
-    def copySheet(self):
-        """Copy sheet"""
+    def duplicateSheet(self):
+        """Make a copy of a sheet"""
 
+        if len(self.sheets) == 0:
+            return
         index = self.tabs.currentIndex()
         name = self.tabs.tabText(index)
-        df = self.sheets[name].table.model.df
+        df = self.sheets[name].table.model.df.copy()
+        meta = self.saveMeta(self.sheets[name])
         new, ok = QInputDialog.getText(self, 'New name', 'Name:',
                     QLineEdit.Normal, name+'_copy')
         if ok:
-            self.addSheet(new, df)
+            self.addSheet(new, df, meta=meta)
         return
 
     def concatSheets(self):
         """Combine sheets into one table"""
 
+        if len(self.sheets) == 0:
+            return
         names = self.sheets.keys()
         if len(names) < 2:
             return
@@ -994,6 +1031,8 @@ class Application(QMainWindow):
     def mergeSheets(self):
         """Merge two sheets"""
 
+        if len(self.sheets) < 2:
+            return
         names = self.sheets.keys()
         opts = {'sheet1':{'type':'combobox','default':'','items':names},
                 'sheet2':{'type':'combobox','default':'','items':names}
@@ -1012,6 +1051,19 @@ class Application(QMainWindow):
             return
         return
 
+    def clearSheets(self, ask=True):
+        """Clear all sheets"""
+
+        if ask == True:
+            reply = QMessageBox.question(self, 'Clear all sheets?',
+                                 'This will remove all sheets. Are you sure?',
+                                 QMessageBox.Yes, QMessageBox.No)
+            if reply == QMessageBox.No:
+                return
+        self.tabs.clear()
+        self.sheets = {}
+        return
+
     def showPlotFrame(self):
         """Show/hide the plot frame"""
 
@@ -1095,11 +1147,11 @@ class Application(QMainWindow):
                 cols = kwds['cols']
                 n = kwds['n']
             if ok:
-                df = data.getSampleData(rows,cols,n)
+                df = util.getSampleData(rows,cols,n)
             else:
                 return
         else:
-            df = data.getPresetData(name)
+            df = util.getPresetData(name)
         self.addSheet(sheetname,df)
         return
 
@@ -1127,13 +1179,6 @@ class Application(QMainWindow):
         self.paste()
         return
 
-    def replot(self):
-        """Plot current"""
-
-        w = self.getCurrentTable()
-        pf = w.pf
-        pf.replot()
-
     def zoomIn(self):
 
         w = self.getCurrentTable()
@@ -1194,18 +1239,20 @@ class Application(QMainWindow):
             self.scratchpad.update(self.scratch_items)
         return
 
-    def plotToScratchpad(self):
+    def plotToScratchpad(self, label=None):
         """Cache the current plot so it can be viewed later"""
 
         w = self.getCurrentTable()
-        index = self.tabs.currentIndex()
-        name = self.tabs.tabText(index)
+        if label == None or label is False:
+            index = self.tabs.currentIndex()
+            name = self.tabs.tabText(index)
+            t = time.strftime("%H:%M:%S")
+            label = name+'-'+t
         #get the current figure and make a copy of it by using pickle
         fig = w.pf.fig
         p = pickle.dumps(fig)
         fig = pickle.loads(p)
-        t = time.strftime("%H:%M:%S")
-        label = name+'-'+t
+
         self.scratch_items[label] = fig
         if hasattr(self, 'scratchpad'):
             self.scratchpad.update(self.scratch_items)
@@ -1238,7 +1285,8 @@ class Application(QMainWindow):
 
         from . import plugin
         default = os.path.join(module_path, 'plugins')
-        paths = [default]
+        other = os.path.join(core.settingspath, 'plugins')
+        paths = [default,other]
         failed = plugin.init_plugin_system(paths)
         self.updatePluginMenu()
         return
@@ -1249,28 +1297,42 @@ class Application(QMainWindow):
         index = self.tabs.currentIndex()
         name = self.tabs.tabText(index)
         tablew = self.sheets[name]
-        if not hasattr(tablew, 'openplugins'):
-            tablew.openplugins = {}
-        openplugins = tablew.openplugins
+        if not hasattr(self, 'openplugins'):
+            self.openplugins = {}
+        openplugins = self.openplugins
 
         if plugin.name in openplugins:
             p = openplugins[plugin.name]
-            p.main.show()
+            self.docks[plugin.name].show()
         else:
-            #plugin should be added to the splitter as a dock widget
-            #should also be able to add standalone windows
             try:
                 p = plugin(parent=self, table=tablew)
                 #track which plugin is running
                 openplugins[plugin.name] = p
             except Exception as e:
                 QMessageBox.information(self, "Plugin error", str(e))
+                return
 
-            if 'docked' in p.capabilities:
-                tablew.splitter.addWidget(p.main)
-                index = tablew.splitter.indexOf(p.main)
-                tablew.splitter.setCollapsible(index, False)
+            #plugin should be added as a dock widget
+            self.showPlugin(p)
+        return
 
+    def showPlugin(self, plugin):
+        """Add plugin as dock widget"""
+
+        dockstyle = '''
+            QDockWidget::title {
+                background-color: #d7edce;
+            }
+        '''
+        dock = QDockWidget(plugin.name)
+        dock.setStyleSheet(dockstyle)
+        area = QScrollArea()
+        area.setWidgetResizable(True)
+        dock.setWidget(area)
+        area.setWidget(plugin.main)
+        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, dock)
+        self.docks[plugin.name] = dock
         return
 
     def updatePluginMenu(self):
@@ -1312,12 +1374,11 @@ class Application(QMainWindow):
         dlg.exec_()
         return
 
-    def online_documentation(self,event=None):
+    def open_url(self,url='',event=None):
         """Open the online documentation"""
 
         import webbrowser
-        link='https://github.com/dmnfarrell/tablexplore'
-        webbrowser.open(link,autoraise=1)
+        webbrowser.open(url,autoraise=1)
         return
 
     def about(self):
