@@ -28,9 +28,15 @@ from pandas.api.types import is_datetime64_any_dtype as is_datetime
 import string
 from .qt import *
 
+homepath = os.path.expanduser("~")
 module_path = os.path.dirname(os.path.abspath(__file__))
 iconpath = os.path.join(module_path, 'icons')
 pluginiconpath = os.path.join(module_path, 'plugins', 'icons')
+settingspath = os.path.join(homepath, '.config','tablexplore')
+if not os.path.exists(settingspath):
+    os.makedirs(settingspath)
+cmapsfile = os.path.join(settingspath, 'cmaps.pkl')
+
 textalignment = None
 MODES = ['default','spreadsheet','locked']
 
@@ -387,6 +393,7 @@ class DataFrameWidget(QWidget):
 
         if self.pf == None:
             self.pf = plotting.PlotViewer(table=self.table, parent=parent)
+            #self.table.selectionModel().selectionChanged.connect(self.pf.updateSeries)
         if parent == None:
             self.pf.show()
         return self.pf
@@ -836,6 +843,31 @@ class DataFrameWidget(QWidget):
         self.refresh()
         return
 
+    def fillStrings(self, column):
+        """Fill column with string data"""
+
+        df = self.table.model.df
+        chartypes = ['lower','upper','printable']
+        opts = {'length': {'type':'spinbox','default':5,'label':'Length'},
+                'chartype':{'type':'combobox','items':chartypes,'default':'lower','label':'Word type'},
+            }
+        dlg = dialogs.MultipleInputDialog(self, opts, title='Fill String', width=300)
+        dlg.exec_()
+        if not dlg.accepted:
+            return
+        kwds = dlg.values
+        namelen = kwds['length']
+        chartype = kwds['chartype']
+        if chartype == 'lower':
+            data = [util.gen_lower(namelen) for i in range(len(df))]
+        elif chartype == 'upper':
+            data = [util.gen_upper(namelen) for i in range(len(df))]
+        else:
+            data = [util.gen_word(namelen) for i in range(len(df))]
+        self.table.storeCurrent()
+        self.table.model.df[column] = data
+        self.refresh()
+
     def fillData(self, column):
         """Fill column with data"""
 
@@ -1243,6 +1275,47 @@ class HeaderProxyStyle(QProxyStyle):
             element, option, painter, widget
         )
 
+class HeaderView(QHeaderView):
+    """"
+    Column header class.
+    """
+    def __init__(self, parent):
+        super(HeaderView, self).__init__(QtCore.Qt.Horizontal, parent)
+        '''self.setStyleSheet(
+            "QHeaderView::section{background-color: #ffffff; "
+            "font-weight: bold; "
+            "border-bottom: 1px solid gray;}")'''
+
+        self.setDefaultAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.Alignment(QtCore.Qt.TextWordWrap))
+        sizePol = QSizePolicy()
+        sizePol.setVerticalPolicy(QSizePolicy.Maximum)
+        sizePol.setHorizontalPolicy(QSizePolicy.Maximum)
+        self.setSizePolicy(sizePol)
+        self.MAX_HEIGHT = 240
+        self.setMinimumHeight(26)
+        self.setMaximumHeight(self.MAX_HEIGHT )
+        self.setSectionsClickable(True)
+        self.setSelectionBehavior(QTableView.SelectColumns)
+        self.setStretchLastSection(False)
+        return
+
+    def sectionSizeFromContents(self, logicalIndex):
+        """Get section size from contents"""
+
+        text = self.model().headerData(logicalIndex, self.orientation(), QtCore.Qt.DisplayRole)
+        alignment = self.defaultAlignment()
+        metrics = QFontMetrics(self.fontMetrics())
+        width = metrics.boundingRect(QtCore.QRect(), alignment, text).width()
+
+        heights = []
+        for i in range(self.count()):
+            text = self.model().headerData(i, self.orientation(),QtCore.Qt.DisplayRole)
+            size = self.sectionSize(i)
+            rect = QtCore.QRect(0, 0, size, self.MAX_HEIGHT)
+            heights.append(metrics.boundingRect(rect, alignment, text).height())
+        height = sorted(heights)[-1] + 5
+        return QtCore.QSize(width, height)
+
 class DataFrameTable(QTableView):
     """
     QTableView with pandas DataFrame as model.
@@ -1259,22 +1332,19 @@ class DataFrameTable(QTableView):
         self.timeformat = timeformat
         self.clicked.connect(self.showSelection)
         #self.doubleClicked.connect(self.handleDoubleClick)
-        #self.setSelectionBehavior(QTableView.SelectRows)
-        #self.setSelectionBehavior(QTableView.SelectColumns)
-        #vh = self.vheader = RowHeader()
-        #self.setVerticalHeader(vh)
+
         vh = self.verticalHeader()
         vh.setVisible(True)
         vh.setDefaultSectionSize(30)
         vh.setMinimumWidth(50)
+        vh.setMaximumWidth(500)
         vh.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         vh.customContextMenuRequested.connect(self.rowHeaderMenu)
-        #vh.sectionClicked.connect(self.rowClicked)
 
+        self.headerview = HeaderView(self)
+        self.setHorizontalHeader(self.headerview)
         hh = self.horizontalHeader()
-        hh.setVisible(True)
 
-        #hh.setSectionResizeMode(QHeaderView.ResizeToContents)
         hh.setDefaultSectionSize(columnwidth)
         hh.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
         hh.customContextMenuRequested.connect(self.columnHeaderMenu)
@@ -1283,9 +1353,8 @@ class DataFrameTable(QTableView):
         hh.setSectionsMovable(True)
         hh.setSelectionBehavior(QTableView.SelectColumns)
         hh.setSelectionMode(QAbstractItemView.ExtendedSelection)
-        hh.setDefaultAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.Alignment(QtCore.Qt.TextWordWrap))
-        #hh.setMaximumHeight(30)
-        hh.setFixedHeight(28)
+        hh.setDefaultAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.Alignment(QtCore.Qt.TextWordWrap))
+
         #formats
         self.setDragEnabled(True)
         self.viewport().setAcceptDrops(True)
@@ -1336,6 +1405,7 @@ class DataFrameTable(QTableView):
         font.setPointSize(int(self.fontsize))
         self.setFont(font)
         self.horizontalHeader().setFont(font)
+        self.verticalHeader().setFont(font)
         return
 
     def refresh(self):
@@ -1585,10 +1655,14 @@ class DataFrameTable(QTableView):
         addColumnAction = colmenu.addAction("Add Column")
         setTypeAction = colmenu.addAction("Set Data Type")
         convertNumericAction = colmenu.addAction("Convert to Numeric")
-
         menu.addAction(colmenu.menuAction())
-        filldataAction = menu.addAction("Fill Data")
-        filldatesAction = menu.addAction("Fill Dates")
+
+        fillmenu = QMenu("Fill",menu)
+        filldataAction = fillmenu.addAction("Fill Data")
+        fillstringsAction = fillmenu.addAction("Fill Strings")
+        filldatesAction = fillmenu.addAction("Fill Dates")
+        menu.addAction(fillmenu.menuAction())
+
         applyFunctionAction = menu.addAction("Apply Function")
         transformResampleAction = menu.addAction("Transform/Resample")
         stringOpAction = menu.addAction("String Operation")
@@ -1616,6 +1690,8 @@ class DataFrameTable(QTableView):
             self.parent.convertDates(column)
         elif action == filldataAction:
             self.parent.fillData(column)
+        elif action == fillstringsAction:
+            self.parent.fillStrings(column)
         elif action == filldatesAction:
             self.parent.fillDates(column)
         elif action == applyFunctionAction:
