@@ -160,6 +160,7 @@ def dialogFromOptions(parent, opts, sections=None,
                     min = int(min)
                     max = int(max)
                     w.setRange(min,max)
+                    w.setMaximum(max)
                     w.setMinimum(min)
                 if 'interval' in opt:
                     w.setSingleStep(opt['interval'])
@@ -208,6 +209,28 @@ def dialogFromOptions(parent, opts, sections=None,
             scol+=1
     return dialog, widgets
 
+def getWidgetValue(w):
+    """Get value from any kind of widget"""
+
+    val = None
+    if type(w) is QLineEdit:
+        val = w.text()
+    elif type(w) is QPlainTextEdit:
+        val = w.toPlainText()
+    elif type(w) is QComboBox or type(w) is QFontComboBox:
+        val = w.currentText()
+    elif type(w) is QListWidget:
+        val = [i.text() for i in w.selectedItems()]
+    elif type(w) is QCheckBox:
+        val = w.isChecked()
+    elif type(w) is QSlider:
+        val = w.value()
+    elif type(w) in [QSpinBox,QDoubleSpinBox]:
+        val = w.value()
+    elif type(w) is ColorButton:
+        val = w.color()
+    return val
+
 def getWidgetValues(widgets):
     """Get values back from a set of widgets"""
 
@@ -216,22 +239,7 @@ def getWidgetValues(widgets):
         val = None
         if i in widgets:
             w = widgets[i]
-            if type(w) is QLineEdit:
-                val = w.text()
-            elif type(w) is QPlainTextEdit:
-                val = w.toPlainText()
-            elif type(w) is QComboBox or type(w) is QFontComboBox:
-                val = w.currentText()
-            elif type(w) is QListWidget:
-                val = [i.text() for i in w.selectedItems()]
-            elif type(w) is QCheckBox:
-                val = w.isChecked()
-            elif type(w) is QSlider:
-                val = w.value()
-            elif type(w) in [QSpinBox,QDoubleSpinBox]:
-                val = w.value()
-            elif type(w) is ColorButton:
-                val = w.color()
+            val = getWidgetValue(w)
             if val != None:
                 kwds[i] = val
     kwds = kwds
@@ -312,7 +320,8 @@ class ColorButton(QPushButton):
         if self._color:
             self.setStyleSheet("background-color: %s;" % self._color)
         else:
-            self.setStyleSheet("")
+            self.setStyleSheet("")    
+        return
 
     def color(self):
         return self._color
@@ -704,6 +713,28 @@ class ComboDelegate(QItemDelegate):
     def currentIndexChanged(self):
         self.commitData.emit(self.sender())
 
+class SimpleDialog(QDialog):
+    """Qdialog for table operations interfaces"""
+    def __init__(self, parent, title=None):
+
+        super(SimpleDialog, self).__init__(parent)
+        self.parent = parent
+        self.setWindowTitle(title)
+        self.setGeometry(QtCore.QRect(400, 300, 1000, 600))
+        self.resize(500, 200)
+        self.layout = QVBoxLayout(self)
+        self.show()
+        return
+
+    def createButtons(self):
+        """Create buttons"""
+        self.button_box = QDialogButtonBox(
+                    QDialogButtonBox.Cancel|QDialogButtonBox.Ok ,
+                    QtCore.Qt.Horizontal,
+                    self
+        )
+        return
+
 class BasicDialog(QDialog):
     """Qdialog for table operations interfaces"""
     def __init__(self, parent, df, title=None, app=None):
@@ -1280,7 +1311,6 @@ class ManageColumnsDialog(BasicDialog):
         self.cols_w.clear()
         self.cols_w.addItems(self.table.model.df.columns)
 
-
 class PreferencesDialog(QDialog):
     """Preferences dialog from config parser options"""
 
@@ -1328,7 +1358,8 @@ class PreferencesDialog(QDialog):
                 'SHOWPLOTTER': {'type':'checkbox','default':bool(options['SHOWPLOTTER']), 'label':'Show Plotter'},
                 'PLOTSTYLE':{'type':'combobox','default':options['PLOTSTYLE'],
                             'items':plotstyles,'label':'Plot Style'},
-                'DPI':{'type':'entry','default':100,'default':options['DPI'], 'label':'Plot DPI'},
+                'DPI':{'type':'entry','default':options['DPI'],#'range':(20,300),'interval':10,
+                        'label':'Plot DPI'},
                 'ICONSIZE':{'type':'spinbox','default':options['ICONSIZE'],'range':(16,64), 'label':'Icon Size'},
                 'THEME':{'type':'combobox','default':options['THEME'],'items': themes,
                         'label': 'Default Theme'}
