@@ -41,14 +41,19 @@ import pandas as pd
 from .qt import *
 from .dialogs import *
 from . import util
+import logging
 
 homepath = os.path.expanduser("~")
 module_path = os.path.dirname(os.path.abspath(__file__))
 iconpath = os.path.join(module_path, 'icons')
+settingspath = os.path.join(homepath, '.config','tablexplore')
+cmapsfile = os.path.join(settingspath, 'cmaps.pkl')
 
 colormaps = sorted(m for m in plt.cm.datad if not m.endswith("_r"))
 markers = ['','o','.','^','v','>','<','s','+','x','p','d','h','*']
-linestyles = ['-','--','-.',':','steps']
+linestyles = ['-','--','-.',':']
+plotkinds = ['line', 'bar', 'barh', 'scatter', 'pie', 'histogram', 'boxplot', 'violinplot', 'dotplot',
+             'heatmap', 'area', 'hexbin', 'scatter_matrix', 'density', 'radviz']
 valid_kwds = {'line': ['alpha', 'colormap', 'grid', 'legend', 'linestyle','ms',
                   'linewidth', 'marker', 'subplots', 'rotx', 'logx', 'logy',
                   'sharex','sharey', 'kind'],
@@ -89,11 +94,33 @@ def defaultOptions():
     opts = {'general': MPLBaseOptions(),
             'format': FormatOptions(),
             'labels': AnnotationOptions(),
-            'axes': AxesOptions(),
-            #'series':SeriesOptions(),
+            'axes': AxesOptions()
             }
     return opts
 
+def update_colormaps():
+    """Load stored colormaps"""
+
+    cmaps = load_colormaps()
+    if cmaps == None:
+        return
+    for name in cmaps:
+        cm=cmaps[name]
+        mpl.colormaps.register(cm)
+        global colormaps
+        colormaps.append(cm.name)
+    return
+
+def load_colormaps():
+
+    if not os.path.exists(cmapsfile):
+        return {}
+    import pickle
+    fp = open(cmapsfile, 'rb')
+    cmaps = pickle.load(fp)
+    fp.close()
+    return cmaps
+
 class PlotWidget(FigureCanvas):
     def __init__(self, parent=None, figure=None, dpi=100, hold=False):
 
@@ -108,11 +135,13 @@ class PlotWidget(FigureCanvas):
 class PlotViewer(QWidget):
     """Plot viewer class"""
     def __init__(self, table, parent=None):
+
         super(PlotViewer, self).__init__(parent)
         self.parent = parent
         self.table = table
         self.createWidgets()
         self.createOptions()
+        #self.seriesopts = SeriesOptions(plotviewer=self)
         self.currentdir = os.path.expanduser('~')
         sizepolicy = QSizePolicy()
         self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
@@ -130,18 +159,84 @@ class PlotViewer(QWidget):
         self.toolbar = NavigationToolbar(self.canvas, layout)
 
         #add custom buttons
-        iconfile = os.path.join(iconpath,'zoom-out.png')
-        a = QAction(QIcon(iconfile), "Reduce",  self)
+        #iconfile = os.path.join(iconpath,'plotseries.png')
+        #a = QAction(QIcon(iconfile), "Customise Series", self)
+        #a.triggered.connect(self.customiseSeries)
+        #self.toolbar.addAction(a)
+        iconfile = os.path.join(iconpath,'reduce.png')
+        a = QAction(QIcon(iconfile), "Reduce elements",  self)
         a.triggered.connect(lambda: self.zoom(zoomin=False))
         self.toolbar.addAction(a)
-        iconfile = os.path.join(iconpath,'zoom-in.png')
-        a = QAction(QIcon(iconfile), "Enlarge",  self)
+        iconfile = os.path.join(iconpath,'enlarge.png')
+        a = QAction(QIcon(iconfile), "Enlarge elements",  self)
         a.triggered.connect(lambda: self.zoom(zoomin=True))
         self.toolbar.addAction(a)
         vbox.addWidget(self.toolbar)
         vbox.addWidget(self.canvas)
         return
 
+    def updateSeries(self, event=None):
+        """Update series with new plots"""
+
+        data = self.table.getSelectedDataFrame()
+        self.seriesopts.series = self.getSeries(data)
+        return
+
+    def customiseSeries(self):
+        """
+        Show custom options for current plot series. Allows plot types to
+        be specified per series and uses a custom plot function.
+        """
+
+        if self.seriesopts.series == {}:
+            self.updateSeries()
+        dlg = self.seriesopts.showDialog(self)
+        dlg.exec_()
+        return
+
+    def customPlot(self):
+        """plot custom series """
+
+        self.applyPlotoptions()
+        self.setStyle()
+        kwds = self.opts['general'].kwds
+        formatkwds = self.opts['format'].kwds
+        kwds.update(formatkwds)
+        axes_layout = kwds['axes_layout']
+        lkwds = self.opts['labels'].kwds.copy()
+        axkwds = self.opts['axes'].kwds
+
+        data = self.table.getSelectedDataFrame()
+        layout='single'
+        self.clear()
+        self.ax = self.fig.add_subplot(111)
+        series = self.seriesopts.series
+        for s in series:
+            df = data[s]
+            skwds = series[s]
+            kind = skwds['kind']
+            #skwds = self.check_kwds(skwds, kind)
+            #print (skwds)
+            df.plot(layout=layout,ax=self.ax,**skwds)
+        self.fig.legend()
+
+        axs=self.ax
+        lkwds.update(kwds)
+        self.setFigureOptions(axs, lkwds)
+        self.canvas.draw()
+        return
+
+    def check_kwds(self, kwds, kind):
+        return dict((k, kwds[k]) for k in valid_kwds[kind] if k in kwds)
+
+    def colorsfromColormap(self, df, cmap):
+        """Column colors from cmap"""
+
+        cols = df.columns
+        clrs = util.gen_colors(cmap,len(cols))
+        colordict = dict(zip(cols,clrs))
+        return colordict
+
     def showTools(self):
         """Show/hide tools dock"""
 
@@ -154,7 +249,6 @@ class PlotViewer(QWidget):
     def createWidgets(self):
         """Create widgets. Plot on left and dock for tools on right."""
 
-        #self.main = QSplitter(Qt.Horizontal, self)
         self.main = QWidget(self)
         hbox = QHBoxLayout(self)
         self.left = left = QWidget(self.main)
@@ -166,20 +260,26 @@ class PlotViewer(QWidget):
     def setFigure(self, figure):
         """Recreate canvas with given figure"""
 
+        self.clear()
         self.canvas.figure = figure
         self.fig = self.canvas.figure
         self.ax = self.canvas.ax
         self.canvas.draw()
         return
 
+    def getFigureSize(self):
+
+        fig=self.fig
+        size = fig.get_size_inches()
+        return size
+
     def createOptions(self):
         """Create option attributes for plotter"""
 
         self.opts = {'general':MPLBaseOptions(),
                     'format':FormatOptions(),
                     'labels':AnnotationOptions(),
-                    'axes':AxesOptions(),
-                    #'series':SeriesOptions()
+                    'axes':AxesOptions()
                     }
         return
 
@@ -232,6 +332,7 @@ class PlotViewer(QWidget):
             self.data = data
 
         self.applyPlotoptions()
+        #self.updateSeries()
         self.setStyle()
         self.plotCurrent()
         return
@@ -251,6 +352,25 @@ class PlotViewer(QWidget):
         self.plot2D(redraw=redraw)
         return
 
+    def getSeries(self, data):
+
+        kwds = self.opts['general'].kwds
+        fkwds = self.opts['format'].kwds
+        kwds.update(fkwds)
+        self.series = series = {}
+        colordict = self.colorsfromColormap(data, fkwds['colormap'])
+        #print (colordict)
+        basekwds = ['kind','marker','ms','linestyle', 'linewidth', 'alpha']
+        skwds = {k:kwds[k] for k in basekwds}
+        #print (fkwds)
+        for c in data.columns:
+            series[c] = skwds.copy()
+            #series[c]['kind'] = kind
+            series[c]['color'] = colordict[c]
+            print (series[c])
+        print (self.series)
+        return series
+
     def applyPlotoptions(self):
         """Apply the current plotter/options"""
 
@@ -631,7 +751,7 @@ class PlotViewer(QWidget):
         rows = int(round(np.sqrt(len(data.columns)),0))
         if len(data.columns) == 1 and kind not in ['pie']:
             kwargs['subplots'] = 0
-        #print (kwargs)
+
         if 'colormap' in kwargs:
             cmap = plt.cm.get_cmap(kwargs['colormap'])
         else:
@@ -755,7 +875,7 @@ class PlotViewer(QWidget):
                 return
             #adjust colormap to avoid white lines
             if cmap != None:
-                cmap = util.adjustColorMap(cmap, 0.15,1.0)
+                #cmap = util.adjustColorMap(cmap, 0.15,1.0)
                 del kwargs['colormap']
             if kind == 'barh':
                 kwargs['xerr']=yerr
@@ -1355,12 +1475,24 @@ class BaseOptions(object):
         self.setWidgetValue(key, new)
         return
 
+    def randomSettings(self):
+        """Get random settings"""
+
+        kwds = self.kwds
+        for k in kwds:
+            opt = self.opts[k]
+            if opt['type'] == 'combobox':
+                kwds[k] = random.choice(opt['items'])
+            elif opt['type'] == 'spinbox':
+                if 'range' in opt.keys():
+                    r=opt['range']
+                    kwds[k] = random.randint(r[0],r[1])
+        return
+
 class MPLBaseOptions(BaseOptions):
     """Class to provide a dialog for matplotlib options and returning
         the selected prefs"""
 
-    kinds = ['line', 'bar', 'barh', 'scatter', 'pie', 'histogram', 'boxplot', 'violinplot', 'dotplot',
-             'heatmap', 'area', 'hexbin', 'scatter_matrix', 'density', 'radviz']
     legendlocs = ['best','upper right','upper left','lower left','lower right','right','center left',
                 'center right','lower center','upper center','center']
 
@@ -1387,7 +1519,7 @@ class MPLBaseOptions(BaseOptions):
                 'sharex':{'type':'checkbox','default':0,'label':'share x'},
                 'sharey':{'type':'checkbox','default':0,'label':'share y'},
                 'legend':{'type':'checkbox','default':1,'label':'legend'},
-                'kind':{'type':'combobox','default':'line','items':self.kinds,'label':'plot type'},
+                'kind':{'type':'combobox','default':'line','items':plotkinds,'label':'plot type'},
                 'stacked':{'type':'checkbox','default':0,'label':'stacked'},
                 'axes_layout':{'type':'combobox','default':'single','items':layouts,'label':'axes layout'},
                 'bins':{'type':'spinbox','default':20,'width':5},
@@ -1421,7 +1553,6 @@ class FormatOptions(BaseOptions):
     def __init__(self):
         """Setup variables"""
 
-        #self.parent = parent
         scales = ['linear','log']
         style_list = ['default', 'classic', 'fivethirtyeight',
                      'seaborn-pastel','seaborn-whitegrid', 'ggplot','bmh',
@@ -1488,7 +1619,7 @@ class AnnotationOptions(BaseOptions):
                 'boxstyle':{'type':'combobox','default':'square','items': bstyles},
                 'text':{'type':'scrolledtext','default':'','width':20},
                 #'align':{'type':'combobox','default':'center','items': alignments},
-                'font':{'type':'combobox','default':defaultfont,'items':fonts},
+                'font':{'type':'font','default':defaultfont,'items':fonts},
                 'fontsize':{'type':'spinbox','default':12,'range':(4,50),'label':'font size'},
                 'fontweight':{'type':'combobox','default':'normal','items': fontweights},
                 'color':{'type':'combobox','default':'black','items': colors},
@@ -1553,44 +1684,124 @@ class AxesOptions(BaseOptions):
 
 class SeriesOptions(BaseOptions):
     """Class for selecting custom plot types for each series"""
-    def __init__(self):
+    def __init__(self, plotviewer):
         """Setup variables"""
 
         super(SeriesOptions, self).__init__()
-        self.groups = grps = OrderedDict({'main':[]})
+        self.series = {}
+        self.plotviewer = plotviewer
         self.setDefaults()
         return
 
-    def updateWidgets(self, parent, df):
-        """Recreate widgets dynamically"""
+    def showDialog(self, parent=None):
 
-        opts = self.opts = {}
-        self.kwds = {}
-        ptypes = ['line','bar','scatter']
-        cols = list(df.columns)
-        cols = [str(c) for c in cols]
-        for name in cols:
-            self.opts[name] = {'type':'combobox','items':ptypes,'default':''}
-        self.groups['main'] = self.opts.keys()
-        dialog, widgets = dialogFromOptions(parent, self.opts, self.groups,
-                                style=self.style)
-        self.widgets = widgets
-        #self.parent.setWidget(dialog)
-        #print (self.widgets)
+        dlg = self.dlg = SimpleDialog(parent, title='Series Options')
+        self.createSeriesWidgets(dlg)
+        btnbox = self.createButtons(dlg)
+        return dlg
+
+    def createButtons(self, parent):
+
+        l = parent.layout
+        bw = self.button_widget = QWidget(parent)
+        vbox = QHBoxLayout(bw)
+        button = QPushButton("Cancel")
+        button.clicked.connect(self.close)
+        vbox.addWidget(button)
+        button = QPushButton("Clear")
+        button.clicked.connect(self.update)
+        vbox.addWidget(button)
+        button = QPushButton("Update")
+        button.clicked.connect(self.update)
+        vbox.addWidget(button)
+        l.addWidget(bw)
         return
 
-    def update(self, df):
-        """Update data widget(s) when dataframe changes"""
+    def createSeriesWidgets(self, parent):
+
+        l = parent.layout
+        l.setAlignment(QtCore.Qt.AlignTop)
+
+        row = QWidget()
+        l.addWidget(row)
+        l2 = QHBoxLayout(row)
+        for i in ['name','kind','color','line style','marker','marker size','alpha']:
+            w=QLabel(i)
+            l2.addWidget(w)
+            #w.setFixedWidth(100)
+        kinds = ['line','bar']
+        self.widgets = {}
+        for s in self.series:
+            self.widgets[s] = {}
+            opt=self.series[s]
+            row = QWidget()
+            l.addWidget(row)
+            l2 = QHBoxLayout(row)
+            w=QLabel(s)
+            w.setFixedWidth(100)
+            l2.addWidget(w)
+            w=QComboBox()
+            w.addItems(kinds)
+            val = self.series[s]['kind']
+            index = w.findText(val)
+            w.setCurrentIndex(index)
+            w.setFixedWidth(80)
+            if val not in ['line','bar']:
+                w.setEnabled(False)
+            l2.addWidget(w)
+            self.widgets[s]['kind'] = w
+            w = ColorButton()
+            val = self.series[s]['color']
+            w.setColor(val)
+            l2.addWidget(w)
+            self.widgets[s]['color'] = w
+            w=QComboBox()
+            w.addItems(linestyles)
+            val = self.series[s]['linestyle']
+            index = w.findText(val)
+            w.setCurrentIndex(index)
+            l2.addWidget(w)
+            self.widgets[s]['linestyle'] = w
+            w=QSpinBox()
+            val = self.series[s]['linewidth']
+            w.setValue(int(val))
+            l2.addWidget(w)
+            self.widgets[s]['linewidth'] = w
+            w=QComboBox()
+            w.addItems(markers)
+            val = self.series[s]['marker']
+            index = w.findText(val)
+            w.setCurrentIndex(index)
+            self.widgets[s]['marker'] = w
+            l2.addWidget(w)
+            w=QSpinBox()
+            val = self.series[s]['ms']
+            w.setValue(int(val))
+            l2.addWidget(w)
+            self.widgets[s]['ms'] = w
+            w=QDoubleSpinBox()
+            val = self.series[s]['alpha']
+            w.setValue(val)
+            w.setRange(0,1)
+            w.setSingleStep(0.1)
+            l2.addWidget(w)
+            self.widgets[s]['alpha'] = w
 
-        if util.check_multiindex(df.columns) == 1:
-            cols = list(df.columns.get_level_values(0))
-        else:
-            cols = list(df.columns)
-        #add empty value
-        cols = [str(c) for c in cols]
-        self.updateWidgets(df)
-        #print (self.widgets)
-        #for name in cols:
-        #    self.widgets[name].clear()
-        #    self.widgets[name].addItems(cols)
+        return
+
+    def update(self, event=None):
+        """Update series options from widgets and replot"""
+
+        for s in self.widgets:
+            for i in self.widgets[s]:
+                w = self.widgets[s][i]
+                val = getWidgetValue(w)
+                #print (s,i,val)
+                self.series[s][i] = val
+        #print (self.series)
+        self.plotviewer.customPlot()
+        return
+
+    def close(self):
+        self.dlg.destroy()
         return
