@@ -27,7 +27,7 @@ from collections import OrderedDict
 from tablexplore.qt import *
 import pandas as pd
 import numpy as np
-from tablexplore import util, data, core, dialogs
+from tablexplore import util, core, dialogs
 from tablexplore.plugin import Plugin
 import pylab as plt
 
@@ -85,7 +85,6 @@ class GISPlugin(Plugin):
             return
         self.parent = parent
         self.tablewidget = table
-        self.ID = 'Simple GIS'
         self.layers = {}
         self.createWidgets()
         self.createMenu()
